#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "eog-aboutme-plugin.h"

#include <string.h> /* For strlen (...) */

#include <gmodule.h>
#include <glib/gi18n-lib.h>
#include <libebook/e-book.h>

#include <eog/eog-debug.h>
#include <eog/eog-thumb-view.h>
#include <eog/eog-image.h>

#define WINDOW_DATA_KEY "EogAboutmeWindowData"
#define MENU_PATH "/MainMenu/ToolsMenu/ToolsOps_3"

EOG_PLUGIN_REGISTER_TYPE(EogAboutmePlugin, eog_aboutme_plugin)

typedef struct
{
	GtkActionGroup  *ui_action_group;
	guint           ui_id;
} WindowData;

typedef struct
{
	EogWindow   *window;
	guint       ui_id;
} UIData;

static void eog_aboutme_plugin_init (EogAboutmePlugin *plugin) {}

static void impl_update_ui (EogPlugin *plugin, EogWindow *window) {}

/* Code taken from Cheese (http://live.gnome.org/Cheese) */
static void
aboutme_cb (GtkAction	*action,
	  		EogWindow 	*window)
{
	GtkWidget 	*thumbview;
	EogImage 	*image;
	EContact  	*contact;
	EBook     	*book;
	GError 		*error;
	GdkPixbuf 	*pixbuf;
	
	thumbview = eog_window_get_thumb_view (window);
	image = eog_thumb_view_get_first_selected_image (EOG_THUMB_VIEW (thumbview));

	if (image) {
    	if (e_book_get_self (&contact, &book, NULL)) {
    	  	gchar *filename = eog_image_get_uri_for_display (image);			
    		  
    	    if (filename) {
    			pixbuf = gdk_pixbuf_new_from_file_at_scale (filename, 150, 150, TRUE, NULL);
    			if (contact) {
    		      	EContactPhoto new_photo;
    				guchar **data;
    				gsize *length;
    	
    				new_photo.type = E_CONTACT_PHOTO_TYPE_INLINED;
    				new_photo.data.inlined.mime_type = "image/jpeg";
    				data = &new_photo.data.inlined.data;
    				length = &new_photo.data.inlined.length;

    				if (gdk_pixbuf_save_to_buffer (pixbuf,
                	     	                      (gchar **) data, 
    											  (gsize *) length,
                	     	                      "png", NULL,
                	     	                      "compression", "9", NULL)) {
    	
    					e_contact_set (contact, E_CONTACT_PHOTO, &new_photo);
    	
    					if (!e_book_commit_contact(book, contact, &error)) {
    						gchar *header;
    						GtkWidget *dlg;
    	
    						header = g_strdup_printf (_("Could not set the Account Photo"));
    	
    						dlg = gtk_message_dialog_new (GTK_WINDOW (window),
                	    						GTK_DIALOG_MODAL |
    								            GTK_DIALOG_DESTROY_WITH_PARENT,
    							                GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
    							                header);
    	
    		    	        gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dlg),
        							            error->message);
    	
        					gtk_dialog_run (GTK_DIALOG (dlg));
    		    	        gtk_widget_destroy (dlg);
    		    	        g_free (header);
        	    	  	}
            		}
    				g_free (*data);	
          		}
    		g_free (filename);
        	}
      	}
    }
}

static const GtkActionEntry action_entries[] =
{
	{ "RunAboutme",
	  "aboutme",
	  "Set as my user image",
	  NULL,
	  "Set as my user image",
	  G_CALLBACK (aboutme_cb) }
};

static void
free_window_data (WindowData *data)
{
	g_return_if_fail (data != NULL);
	eog_debug (DEBUG_PLUGINS);
	g_object_unref (data->ui_action_group);
	g_free (data);
}

static void
ui_show (GtkWidget *widget ,
         gpointer   data)
{

    EogWindow       *window;
	GtkWidget 	    *thumbview;
	EogImage 	    *image;
    GtkUIManager    *manager;
    UIData          *ui_data = data;

    window = ui_data->window;
	thumbview = eog_window_get_thumb_view (window);
	image = eog_thumb_view_get_first_selected_image (EOG_THUMB_VIEW (thumbview));
   	manager = eog_window_get_ui_manager (window);

    if (image) {	
    	gtk_ui_manager_add_ui (manager, 
    			       ui_data->ui_id, 
    			       MENU_PATH,
    			       "RunAboutme", 
    			       "RunAboutme",
    			       GTK_UI_MANAGER_MENUITEM, 
    			       FALSE);
    }
    else {
	    gtk_ui_manager_remove_ui (manager,
		                		  ui_data->ui_id);
    }
}

static void
eog_aboutme_plugin_finalize (GObject *object)
{
	G_OBJECT_CLASS (eog_aboutme_plugin_parent_class)->finalize (object);
}

static void
impl_activate (EogPlugin *plugin,
	       EogWindow *window)
{
	GtkUIManager    *manager;
	GtkWidget 	    *thumbview;
    WindowData      *data;
    UIData          *ui_data;  
	
	eog_debug (DEBUG_PLUGINS);

	data = g_new (WindowData, 1);
	manager = eog_window_get_ui_manager (window);

	data->ui_action_group = gtk_action_group_new ("EogAboutmePluginActions");
	
	gtk_action_group_set_translation_domain (data->ui_action_group, 
						 GETTEXT_PACKAGE);
	gtk_action_group_add_actions (data->ui_action_group,
				      action_entries,
				      G_N_ELEMENTS (action_entries),
				      window);

	gtk_ui_manager_insert_action_group (manager,
					    data->ui_action_group,
					    -1);

	data->ui_id = gtk_ui_manager_new_merge_id (manager);

	g_object_set_data_full (G_OBJECT (window), 
				WINDOW_DATA_KEY, 
				data,
				(GDestroyNotify) free_window_data);

	thumbview = eog_window_get_thumb_view (window);

    /* Listen to image changes */
    ui_data = g_new (UIData, 1);
    ui_data->window = window;
    ui_data->ui_id = data->ui_id;
    g_signal_connect (G_OBJECT (thumbview), "selection_changed", 
			  G_CALLBACK (ui_show), ui_data);
    ui_show(NULL, ui_data);
}

static void
impl_deactivate	(EogPlugin *plugin,
		 EogWindow *window)
{
	GtkUIManager    *manager;
	WindowData      *data;

	eog_debug (DEBUG_PLUGINS);

	manager = eog_window_get_ui_manager (window);

	data = (WindowData *) g_object_get_data (G_OBJECT (window),
						 WINDOW_DATA_KEY);
	g_return_if_fail (data != NULL);

	gtk_ui_manager_remove_ui (manager,
				  data->ui_id);

	gtk_ui_manager_remove_action_group (manager,
					    data->ui_action_group);

	g_object_set_data (G_OBJECT (window),
			   WINDOW_DATA_KEY,
			   NULL);
}

static void
eog_aboutme_plugin_class_init (EogAboutmePluginClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);
	EogPluginClass *plugin_class = EOG_PLUGIN_CLASS (klass);
	object_class->finalize = eog_aboutme_plugin_finalize;
	plugin_class->activate = impl_activate;
	plugin_class->deactivate = impl_deactivate;
	plugin_class->update_ui = impl_update_ui;
}

